/*
 *  Blocks-10.0.0.txt
 *  Date: 2017-04-12, 17:30:00 GMT [KW]
 *  (C) 2017 Unicode(R), Inc.
 *  For terms of use, see http://www.unicode.org/terms_of_use.html
 * 
 *  Unicode Character Database
 *  For documentation, see http://www.unicode.org/reports/tr44/
 * 
 *  Format:
 *  Start Code..End Code; Block Name

 *  ================================================

 *  Note:   When comparing block names, casing, whitespace, hyphens,
 *          and underbars are ignored.
 *          For example, "Latin Extended-A" and "latin extended a" are equivalent.
 *          For more information on the comparison of property values,
 *             see UAX  * 44: http://www.unicode.org/reports/tr44/
 * 
 *   All block ranges start with a value where (cp MOD 16) = 0,
 *   and end with a value where (cp MOD 16) = 15. In other words,
 *   the last hexadecimal digit of the start of range is ...0
 *   and the last hexadecimal digit of the end of range is ...F.
 *   This constraint on block ranges guarantees that allocations
 *   are done in terms of whole columns, and that code chart display
 *   never involves splitting columns in the charts.
 * 
 *   All code points not explicitly listed for Block
 *   have the value No_Block.
 */

struct ucd_block {
	uint32_t first;
	uint32_t last;
	const char *name;
};

static struct ucd_block const ucd_blocks[] = {
	{ 0x000000, 0x00007F, "Basic Latin" },
	{ 0x000080, 0x0000FF, "Latin-1 Supplement" },
	{ 0x000100, 0x00017F, "Latin Extended-A" },
	{ 0x000180, 0x00024F, "Latin Extended-B" },
	{ 0x000250, 0x0002AF, "IPA Extensions" },
	{ 0x0002B0, 0x0002FF, "Spacing Modifier Letters" },
	{ 0x000300, 0x00036F, "Combining Diacritical Marks" },
	{ 0x000370, 0x0003FF, "Greek and Coptic" },
	{ 0x000400, 0x0004FF, "Cyrillic" },
	{ 0x000500, 0x00052F, "Cyrillic Supplement" },
	{ 0x000530, 0x00058F, "Armenian" },
	{ 0x000590, 0x0005FF, "Hebrew" },
	{ 0x000600, 0x0006FF, "Arabic" },
	{ 0x000700, 0x00074F, "Syriac" },
	{ 0x000750, 0x00077F, "Arabic Supplement" },
	{ 0x000780, 0x0007BF, "Thaana" },
	{ 0x0007C0, 0x0007FF, "NKo" },
	{ 0x000800, 0x00083F, "Samaritan" },
	{ 0x000840, 0x00085F, "Mandaic" },
	{ 0x000860, 0x00086F, "Syriac Supplement" },
	{ 0x0008A0, 0x0008FF, "Arabic Extended-A" },
	{ 0x000900, 0x00097F, "Devanagari" },
	{ 0x000980, 0x0009FF, "Bengali" },
	{ 0x000A00, 0x000A7F, "Gurmukhi" },
	{ 0x000A80, 0x000AFF, "Gujarati" },
	{ 0x000B00, 0x000B7F, "Oriya" },
	{ 0x000B80, 0x000BFF, "Tamil" },
	{ 0x000C00, 0x000C7F, "Telugu" },
	{ 0x000C80, 0x000CFF, "Kannada" },
	{ 0x000D00, 0x000D7F, "Malayalam" },
	{ 0x000D80, 0x000DFF, "Sinhala" },
	{ 0x000E00, 0x000E7F, "Thai" },
	{ 0x000E80, 0x000EFF, "Lao" },
	{ 0x000F00, 0x000FFF, "Tibetan" },
	{ 0x001000, 0x00109F, "Myanmar" },
	{ 0x0010A0, 0x0010FF, "Georgian" },
	{ 0x001100, 0x0011FF, "Hangul Jamo" },
	{ 0x001200, 0x00137F, "Ethiopic" },
	{ 0x001380, 0x00139F, "Ethiopic Supplement" },
	{ 0x0013A0, 0x0013FF, "Cherokee" },
	{ 0x001400, 0x00167F, "Unified Canadian Aboriginal Syllabics" },
	{ 0x001680, 0x00169F, "Ogham" },
	{ 0x0016A0, 0x0016FF, "Runic" },
	{ 0x001700, 0x00171F, "Tagalog" },
	{ 0x001720, 0x00173F, "Hanunoo" },
	{ 0x001740, 0x00175F, "Buhid" },
	{ 0x001760, 0x00177F, "Tagbanwa" },
	{ 0x001780, 0x0017FF, "Khmer" },
	{ 0x001800, 0x0018AF, "Mongolian" },
	{ 0x0018B0, 0x0018FF, "Unified Canadian Aboriginal Syllabics Extended" },
	{ 0x001900, 0x00194F, "Limbu" },
	{ 0x001950, 0x00197F, "Tai Le" },
	{ 0x001980, 0x0019DF, "New Tai Lue" },
	{ 0x0019E0, 0x0019FF, "Khmer Symbols" },
	{ 0x001A00, 0x001A1F, "Buginese" },
	{ 0x001A20, 0x001AAF, "Tai Tham" },
	{ 0x001AB0, 0x001AFF, "Combining Diacritical Marks Extended" },
	{ 0x001B00, 0x001B7F, "Balinese" },
	{ 0x001B80, 0x001BBF, "Sundanese" },
	{ 0x001BC0, 0x001BFF, "Batak" },
	{ 0x001C00, 0x001C4F, "Lepcha" },
	{ 0x001C50, 0x001C7F, "Ol Chiki" },
	{ 0x001C80, 0x001C8F, "Cyrillic Extended-C" },
	{ 0x001CC0, 0x001CCF, "Sundanese Supplement" },
	{ 0x001CD0, 0x001CFF, "Vedic Extensions" },
	{ 0x001D00, 0x001D7F, "Phonetic Extensions" },
	{ 0x001D80, 0x001DBF, "Phonetic Extensions Supplement" },
	{ 0x001DC0, 0x001DFF, "Combining Diacritical Marks Supplement" },
	{ 0x001E00, 0x001EFF, "Latin Extended Additional" },
	{ 0x001F00, 0x001FFF, "Greek Extended" },
	{ 0x002000, 0x00206F, "General Punctuation" },
	{ 0x002070, 0x00209F, "Superscripts and Subscripts" },
	{ 0x0020A0, 0x0020CF, "Currency Symbols" },
	{ 0x0020D0, 0x0020FF, "Combining Diacritical Marks for Symbols" },
	{ 0x002100, 0x00214F, "Letterlike Symbols" },
	{ 0x002150, 0x00218F, "Number Forms" },
	{ 0x002190, 0x0021FF, "Arrows" },
	{ 0x002200, 0x0022FF, "Mathematical Operators" },
	{ 0x002300, 0x0023FF, "Miscellaneous Technical" },
	{ 0x002400, 0x00243F, "Control Pictures" },
	{ 0x002440, 0x00245F, "Optical Character Recognition" },
	{ 0x002460, 0x0024FF, "Enclosed Alphanumerics" },
	{ 0x002500, 0x00257F, "Box Drawing" },
	{ 0x002580, 0x00259F, "Block Elements" },
	{ 0x0025A0, 0x0025FF, "Geometric Shapes" },
	{ 0x002600, 0x0026FF, "Miscellaneous Symbols" },
	{ 0x002700, 0x0027BF, "Dingbats" },
	{ 0x0027C0, 0x0027EF, "Miscellaneous Mathematical Symbols-A" },
	{ 0x0027F0, 0x0027FF, "Supplemental Arrows-A" },
	{ 0x002800, 0x0028FF, "Braille Patterns" },
	{ 0x002900, 0x00297F, "Supplemental Arrows-B" },
	{ 0x002980, 0x0029FF, "Miscellaneous Mathematical Symbols-B" },
	{ 0x002A00, 0x002AFF, "Supplemental Mathematical Operators" },
	{ 0x002B00, 0x002BFF, "Miscellaneous Symbols and Arrows" },
	{ 0x002C00, 0x002C5F, "Glagolitic" },
	{ 0x002C60, 0x002C7F, "Latin Extended-C" },
	{ 0x002C80, 0x002CFF, "Coptic" },
	{ 0x002D00, 0x002D2F, "Georgian Supplement" },
	{ 0x002D30, 0x002D7F, "Tifinagh" },
	{ 0x002D80, 0x002DDF, "Ethiopic Extended" },
	{ 0x002DE0, 0x002DFF, "Cyrillic Extended-A" },
	{ 0x002E00, 0x002E7F, "Supplemental Punctuation" },
	{ 0x002E80, 0x002EFF, "CJK Radicals Supplement" },
	{ 0x002F00, 0x002FDF, "Kangxi Radicals" },
	{ 0x002FF0, 0x002FFF, "Ideographic Description Characters" },
	{ 0x003000, 0x00303F, "CJK Symbols and Punctuation" },
	{ 0x003040, 0x00309F, "Hiragana" },
	{ 0x0030A0, 0x0030FF, "Katakana" },
	{ 0x003100, 0x00312F, "Bopomofo" },
	{ 0x003130, 0x00318F, "Hangul Compatibility Jamo" },
	{ 0x003190, 0x00319F, "Kanbun" },
	{ 0x0031A0, 0x0031BF, "Bopomofo Extended" },
	{ 0x0031C0, 0x0031EF, "CJK Strokes" },
	{ 0x0031F0, 0x0031FF, "Katakana Phonetic Extensions" },
	{ 0x003200, 0x0032FF, "Enclosed CJK Letters and Months" },
	{ 0x003300, 0x0033FF, "CJK Compatibility" },
	{ 0x003400, 0x004DBF, "CJK Unified Ideographs Extension A" },
	{ 0x004DC0, 0x004DFF, "Yijing Hexagram Symbols" },
	{ 0x004E00, 0x009FFF, "CJK Unified Ideographs" },
	{ 0x00A000, 0x00A48F, "Yi Syllables" },
	{ 0x00A490, 0x00A4CF, "Yi Radicals" },
	{ 0x00A4D0, 0x00A4FF, "Lisu" },
	{ 0x00A500, 0x00A63F, "Vai" },
	{ 0x00A640, 0x00A69F, "Cyrillic Extended-B" },
	{ 0x00A6A0, 0x00A6FF, "Bamum" },
	{ 0x00A700, 0x00A71F, "Modifier Tone Letters" },
	{ 0x00A720, 0x00A7FF, "Latin Extended-D" },
	{ 0x00A800, 0x00A82F, "Syloti Nagri" },
	{ 0x00A830, 0x00A83F, "Common Indic Number Forms" },
	{ 0x00A840, 0x00A87F, "Phags-pa" },
	{ 0x00A880, 0x00A8DF, "Saurashtra" },
	{ 0x00A8E0, 0x00A8FF, "Devanagari Extended" },
	{ 0x00A900, 0x00A92F, "Kayah Li" },
	{ 0x00A930, 0x00A95F, "Rejang" },
	{ 0x00A960, 0x00A97F, "Hangul Jamo Extended-A" },
	{ 0x00A980, 0x00A9DF, "Javanese" },
	{ 0x00A9E0, 0x00A9FF, "Myanmar Extended-B" },
	{ 0x00AA00, 0x00AA5F, "Cham" },
	{ 0x00AA60, 0x00AA7F, "Myanmar Extended-A" },
	{ 0x00AA80, 0x00AADF, "Tai Viet" },
	{ 0x00AAE0, 0x00AAFF, "Meetei Mayek Extensions" },
	{ 0x00AB00, 0x00AB2F, "Ethiopic Extended-A" },
	{ 0x00AB30, 0x00AB6F, "Latin Extended-E" },
	{ 0x00AB70, 0x00ABBF, "Cherokee Supplement" },
	{ 0x00ABC0, 0x00ABFF, "Meetei Mayek" },
	{ 0x00AC00, 0x00D7AF, "Hangul Syllables" },
	{ 0x00D7B0, 0x00D7FF, "Hangul Jamo Extended-B" },
	{ 0x00D800, 0x00DB7F, "High Surrogates" },
	{ 0x00DB80, 0x00DBFF, "High Private Use Surrogates" },
	{ 0x00DC00, 0x00DFFF, "Low Surrogates" },
	{ 0x00E000, 0x00F8FF, "Private Use Area" },
	{ 0x00F900, 0x00FAFF, "CJK Compatibility Ideographs" },
	{ 0x00FB00, 0x00FB4F, "Alphabetic Presentation Forms" },
	{ 0x00FB50, 0x00FDFF, "Arabic Presentation Forms-A" },
	{ 0x00FE00, 0x00FE0F, "Variation Selectors" },
	{ 0x00FE10, 0x00FE1F, "Vertical Forms" },
	{ 0x00FE20, 0x00FE2F, "Combining Half Marks" },
	{ 0x00FE30, 0x00FE4F, "CJK Compatibility Forms" },
	{ 0x00FE50, 0x00FE6F, "Small Form Variants" },
	{ 0x00FE70, 0x00FEFF, "Arabic Presentation Forms-B" },
	{ 0x00FF00, 0x00FFEF, "Halfwidth and Fullwidth Forms" },
	{ 0x00FFF0, 0x00FFFF, "Specials" },
	{ 0x010000, 0x01007F, "Linear B Syllabary" },
	{ 0x010080, 0x0100FF, "Linear B Ideograms" },
	{ 0x010100, 0x01013F, "Aegean Numbers" },
	{ 0x010140, 0x01018F, "Ancient Greek Numbers" },
	{ 0x010190, 0x0101CF, "Ancient Symbols" },
	{ 0x0101D0, 0x0101FF, "Phaistos Disc" },
	{ 0x010280, 0x01029F, "Lycian" },
	{ 0x0102A0, 0x0102DF, "Carian" },
	{ 0x0102E0, 0x0102FF, "Coptic Epact Numbers" },
	{ 0x010300, 0x01032F, "Old Italic" },
	{ 0x010330, 0x01034F, "Gothic" },
	{ 0x010350, 0x01037F, "Old Permic" },
	{ 0x010380, 0x01039F, "Ugaritic" },
	{ 0x0103A0, 0x0103DF, "Old Persian" },
	{ 0x010400, 0x01044F, "Deseret" },
	{ 0x010450, 0x01047F, "Shavian" },
	{ 0x010480, 0x0104AF, "Osmanya" },
	{ 0x0104B0, 0x0104FF, "Osage" },
	{ 0x010500, 0x01052F, "Elbasan" },
	{ 0x010530, 0x01056F, "Caucasian Albanian" },
	{ 0x010600, 0x01077F, "Linear A" },
	{ 0x010800, 0x01083F, "Cypriot Syllabary" },
	{ 0x010840, 0x01085F, "Imperial Aramaic" },
	{ 0x010860, 0x01087F, "Palmyrene" },
	{ 0x010880, 0x0108AF, "Nabataean" },
	{ 0x0108E0, 0x0108FF, "Hatran" },
	{ 0x010900, 0x01091F, "Phoenician" },
	{ 0x010920, 0x01093F, "Lydian" },
	{ 0x010980, 0x01099F, "Meroitic Hieroglyphs" },
	{ 0x0109A0, 0x0109FF, "Meroitic Cursive" },
	{ 0x010A00, 0x010A5F, "Kharoshthi" },
	{ 0x010A60, 0x010A7F, "Old South Arabian" },
	{ 0x010A80, 0x010A9F, "Old North Arabian" },
	{ 0x010AC0, 0x010AFF, "Manichaean" },
	{ 0x010B00, 0x010B3F, "Avestan" },
	{ 0x010B40, 0x010B5F, "Inscriptional Parthian" },
	{ 0x010B60, 0x010B7F, "Inscriptional Pahlavi" },
	{ 0x010B80, 0x010BAF, "Psalter Pahlavi" },
	{ 0x010C00, 0x010C4F, "Old Turkic" },
	{ 0x010C80, 0x010CFF, "Old Hungarian" },
	{ 0x010E60, 0x010E7F, "Rumi Numeral Symbols" },
	{ 0x011000, 0x01107F, "Brahmi" },
	{ 0x011080, 0x0110CF, "Kaithi" },
	{ 0x0110D0, 0x0110FF, "Sora Sompeng" },
	{ 0x011100, 0x01114F, "Chakma" },
	{ 0x011150, 0x01117F, "Mahajani" },
	{ 0x011180, 0x0111DF, "Sharada" },
	{ 0x0111E0, 0x0111FF, "Sinhala Archaic Numbers" },
	{ 0x011200, 0x01124F, "Khojki" },
	{ 0x011280, 0x0112AF, "Multani" },
	{ 0x0112B0, 0x0112FF, "Khudawadi" },
	{ 0x011300, 0x01137F, "Grantha" },
	{ 0x011400, 0x01147F, "Newa" },
	{ 0x011480, 0x0114DF, "Tirhuta" },
	{ 0x011580, 0x0115FF, "Siddham" },
	{ 0x011600, 0x01165F, "Modi" },
	{ 0x011660, 0x01167F, "Mongolian Supplement" },
	{ 0x011680, 0x0116CF, "Takri" },
	{ 0x011700, 0x01173F, "Ahom" },
	{ 0x0118A0, 0x0118FF, "Warang Citi" },
	{ 0x011A00, 0x011A4F, "Zanabazar Square" },
	{ 0x011A50, 0x011AAF, "Soyombo" },
	{ 0x011AC0, 0x011AFF, "Pau Cin Hau" },
	{ 0x011C00, 0x011C6F, "Bhaiksuki" },
	{ 0x011C70, 0x011CBF, "Marchen" },
	{ 0x011D00, 0x011D5F, "Masaram Gondi" },
	{ 0x012000, 0x0123FF, "Cuneiform" },
	{ 0x012400, 0x01247F, "Cuneiform Numbers and Punctuation" },
	{ 0x012480, 0x01254F, "Early Dynastic Cuneiform" },
	{ 0x013000, 0x01342F, "Egyptian Hieroglyphs" },
	{ 0x014400, 0x01467F, "Anatolian Hieroglyphs" },
	{ 0x016800, 0x016A3F, "Bamum Supplement" },
	{ 0x016A40, 0x016A6F, "Mro" },
	{ 0x016AD0, 0x016AFF, "Bassa Vah" },
	{ 0x016B00, 0x016B8F, "Pahawh Hmong" },
	{ 0x016F00, 0x016F9F, "Miao" },
	{ 0x016FE0, 0x016FFF, "Ideographic Symbols and Punctuation" },
	{ 0x017000, 0x0187FF, "Tangut" },
	{ 0x018800, 0x018AFF, "Tangut Components" },
	{ 0x01B000, 0x01B0FF, "Kana Supplement" },
	{ 0x01B100, 0x01B12F, "Kana Extended-A" },
	{ 0x01B170, 0x01B2FF, "Nushu" },
	{ 0x01BC00, 0x01BC9F, "Duployan" },
	{ 0x01BCA0, 0x01BCAF, "Shorthand Format Controls" },
	{ 0x01D000, 0x01D0FF, "Byzantine Musical Symbols" },
	{ 0x01D100, 0x01D1FF, "Musical Symbols" },
	{ 0x01D200, 0x01D24F, "Ancient Greek Musical Notation" },
	{ 0x01D300, 0x01D35F, "Tai Xuan Jing Symbols" },
	{ 0x01D360, 0x01D37F, "Counting Rod Numerals" },
	{ 0x01D400, 0x01D7FF, "Mathematical Alphanumeric Symbols" },
	{ 0x01D800, 0x01DAAF, "Sutton SignWriting" },
	{ 0x01E000, 0x01E02F, "Glagolitic Supplement" },
	{ 0x01E800, 0x01E8DF, "Mende Kikakui" },
	{ 0x01E900, 0x01E95F, "Adlam" },
	{ 0x01EE00, 0x01EEFF, "Arabic Mathematical Alphabetic Symbols" },
	{ 0x01F000, 0x01F02F, "Mahjong Tiles" },
	{ 0x01F030, 0x01F09F, "Domino Tiles" },
	{ 0x01F0A0, 0x01F0FF, "Playing Cards" },
	{ 0x01F100, 0x01F1FF, "Enclosed Alphanumeric Supplement" },
	{ 0x01F200, 0x01F2FF, "Enclosed Ideographic Supplement" },
	{ 0x01F300, 0x01F5FF, "Miscellaneous Symbols and Pictographs" },
	{ 0x01F600, 0x01F64F, "Emoticons" },
	{ 0x01F650, 0x01F67F, "Ornamental Dingbats" },
	{ 0x01F680, 0x01F6FF, "Transport and Map Symbols" },
	{ 0x01F700, 0x01F77F, "Alchemical Symbols" },
	{ 0x01F780, 0x01F7FF, "Geometric Shapes Extended" },
	{ 0x01F800, 0x01F8FF, "Supplemental Arrows-C" },
	{ 0x01F900, 0x01F9FF, "Supplemental Symbols and Pictographs" },
	{ 0x020000, 0x02A6DF, "CJK Unified Ideographs Extension B" },
	{ 0x02A700, 0x02B73F, "CJK Unified Ideographs Extension C" },
	{ 0x02B740, 0x02B81F, "CJK Unified Ideographs Extension D" },
	{ 0x02B820, 0x02CEAF, "CJK Unified Ideographs Extension E" },
	{ 0x02CEB0, 0x02EBEF, "CJK Unified Ideographs Extension F" },
	{ 0x02F800, 0x02FA1F, "CJK Compatibility Ideographs Supplement" },
	{ 0x0E0000, 0x0E007F, "Tags" },
	{ 0x0E0100, 0x0E01EF, "Variation Selectors Supplement" },
	{ 0x0F0000, 0x0FFFFF, "Supplementary Private Use Area-A" },
	{ 0x100000, 0x10FFFF, "Supplementary Private Use Area-B" }
};
