/*
Copyright (c) 1998 by Juliusz Chroboczek

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

/* These data are very dodgy. */

#include "linux/libcwrap.h"
#include "bics2uni.h"
#include "verify.h"

/*
 * BICS -> UNICODE translation containing -1 for characters
 * with bics indexes to be picked from the default mapping
 * (table below, see the ft2.c ft2_load_glyph() algorithm).
 *
 * Note that this table is indexed by BICS character Index
 * rather than BICS character ID.
 */
unsigned short const Bics2Unicode[BICS_COUNT] = {
/*   0 ID 0020 */	0x0020,
/*   1 ID 0021 */	0x0021,
/*   2 ID 0022 */	0x0022,
/*   3 ID 0023 */	0x0023,
/*   4 ID 0024 */	0x0024,
/*   5 ID 0025 */	0x0025,
/*   6 ID 0026 */	0x0026,
/*   7 ID 0027 */	0x0027,
/*   8 ID 0028 */	0x0028,
/*   9 ID 0029 */	0x0029,
/*  10 ID 002a */	0x002a,
/*  11 ID 002b */	0x002b,
/*  12 ID 002c */	0x002c,
/*  13 ID 002d */	0x002d,
/*  14 ID 002e */	0x002e,
/*  15 ID 002f */	0x002f,
/*  16 ID 0030 */	0x0030,
/*  17 ID 0031 */	0x0031,
/*  18 ID 0032 */	0x0032,
/*  19 ID 0033 */	0x0033,
/*  20 ID 0034 */	0x0034,
/*  21 ID 0035 */	0x0035,
/*  22 ID 0036 */	0x0036,
/*  23 ID 0037 */	0x0037,
/*  24 ID 0038 */	0x0038,
/*  25 ID 0039 */	0x0039,
/*  26 ID 003a */	0x003a,
/*  27 ID 003b */	0x003b,
/*  28 ID 003c */	0x003c,
/*  29 ID 003d */	0x003d,
/*  30 ID 003e */	0x003e,
/*  31 ID 003f */	0x003f,
/*  32 ID 0040 */	0x0040,
/*  33 ID 0041 */	0x0041,
/*  34 ID 0042 */	0x0042,
/*  35 ID 0043 */	0x0043,
/*  36 ID 0044 */	0x0044,
/*  37 ID 0045 */	0x0045,
/*  38 ID 0046 */	0x0046,
/*  39 ID 0047 */	0x0047,
/*  40 ID 0048 */	0x0048,
/*  41 ID 0049 */	0x0049,
/*  42 ID 004a */	0x004a,
/*  43 ID 004b */	0x004b,
/*  44 ID 004c */	0x004c,
/*  45 ID 004d */	0x004d,
/*  46 ID 004e */	0x004e,
/*  47 ID 004f */	0x004f,
/*  48 ID 0050 */	0x0050,
/*  49 ID 0051 */	0x0051,
/*  50 ID 0052 */	0x0052,
/*  51 ID 0053 */	0x0053,
/*  52 ID 0054 */	0x0054,
/*  53 ID 0055 */	0x0055,
/*  54 ID 0056 */	0x0056,
/*  55 ID 0057 */	0x0057,
/*  56 ID 0058 */	0x0058,
/*  57 ID 0059 */	0x0059,
/*  58 ID 005a */	0x005a,
/*  59 ID 005b */	0x005b,
/*  60 ID 005c */	0x005c,
/*  61 ID 005d */	0x005d,
/*  62 ID 005e */	0x005e,
/*  63 ID 005f */	0x005f,
/*  64 ID 0060 */	0x0060,
/*  65 ID 0061 */	0x0061,
/*  66 ID 0062 */	0x0062,
/*  67 ID 0063 */	0x0063,
/*  68 ID 0064 */	0x0064,
/*  69 ID 0065 */	0x0065,
/*  70 ID 0066 */	0x0066,
/*  71 ID 0067 */	0x0067,
/*  72 ID 0068 */	0x0068,
/*  73 ID 0069 */	0x0069,
/*  74 ID 006a */	0x006a,
/*  75 ID 006b */	0x006b,
/*  76 ID 006c */	0x006c,
/*  77 ID 006d */	0x006d,
/*  78 ID 006e */	0x006e,
/*  79 ID 006f */	0x006f,
/*  80 ID 0070 */	0x0070,
/*  81 ID 0071 */	0x0071,
/*  82 ID 0072 */	0x0072,
/*  83 ID 0073 */	0x0073,
/*  84 ID 0074 */	0x0074,
/*  85 ID 0075 */	0x0075,
/*  86 ID 0076 */	0x0076,
/*  87 ID 0077 */	0x0077,
/*  88 ID 0078 */	0x0078,
/*  89 ID 0079 */	0x0079,
/*  90 ID 007a */	0x007a,
/*  91 ID 007b */	0x007b,
/*  92 ID 007c */	0x007c,
/*  93 ID 007d */	0x007d,
/*  94 ID 007e */	0x007e,
/*  95 ID 0081 */	0xfb01, /* LATIN SMALL LIGATURE FI */
/*  96 ID 0082 */	0xfb02, /* LATIN SMALL LIGATURE FL */
/*  97 ID 0083 */	0x00a3,	/* POUND SIGN */
/*  98 ID 0084 */	0x00a2,	/* CENT SIGN */
/*  99 ID 0085 */	0x0192, /* LATIN SMALL LETTER F WITH HOOK */
/* 100 ID 0086 */	0x2030, /* PER MILLE SIGN */
/* 101 ID 0087 */	0x2215, /* DIVISION SLASH */
/* 102 ID 0088 */	0x00b7, /* MIDDLE DOT */
/* 103 ID 0089 */	0x2018, /* LEFT SINGLE QUOTATION MARK */
/* 104 ID 008a */	0x201c, /* LEFT DOUBLE QUOTATION MARK */
/* 105 ID 008b */	0x201d, /* DOUBLE HIGH QUOTATION MARK */
/* 106 ID 008c */	0x201a, /* SINGLE LOW QUOTATION MARK */
/* 107 ID 008d */	0x201e, /* DOUBLE LOW-9 QUOTATION MARK */
/* 108 ID 008e */	0x2020, /* DAGGER */
/* 109 ID 008f */	0x2021, /* DOUBLE DAGGER */
/* 110 ID 0090 */	0x00a7, /* SECTION SIGN */
/* 111 ID 0091 */	0x2013, /* EN DASH */
/* 112 ID 0092 */	0x2014, /* EM DASH */
/* 113 ID 0093 */	0x00c5, /* LATIN CAPITAL LETTER A WITH RING ABOVE */
/* 114 ID 0094 */	0x00c6, /* LATIN CAPITAL LETTER AE */
/* 115 ID 0095 */	0x00d8, /* LATIN CAPITAL LETTER O WITH STROKE */
/* 116 ID 0096 */	0x0152, /* LATIN CAPITAL LIGATURE OE */
/* 117 ID 0097 */	0x00e5, /* LATIN SMALL LETTER A WITH RING ABOVE */
/* 118 ID 0098 */	0x00e6, /* LATIN SMALL LETTER AE */
/* 119 ID 0099 */	0x00f8, /* LATIN SMALL LETTER O WITH STROKE */
/* 120 ID 009a */	0x0153, /* LATIN SMALL LIGATURE OE */
/* 121 ID 009b */	0x00df, /* LATIN SMALL LETTER SHARP S */
/* 122 ID 009c */	0x0131, /* LATIN SMALL LETTER DOTLESS I */
/* 123 ID 009d */	0x2039, /* SINGLE LEFT-POINTING ANGLE QUOTATION MARK */
/* 124 ID 009e */	0x203a, /* SINGLE RIGHT-POINTING ANGLE QUOTATION MARK */
/* 125 ID 009f */	0x00ab, /* LEFT-POINTING DOUBLE ANGLE QUOTATION MARK */
/* 126 ID 00a0 */	0x00bb, /* RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK */
/* 127 ID 00a1 */	0x00bf, /* INVERTED QUESTION MARK */
/* 128 ID 00a2 */	0x00a1, /* INVERTED EXCLAMATION MARK */
/* 129 ID 00a3 */	0x00b4, /* ACUTE ACCENT */
/* 130 ID 00a4 */	0x02ca, /* MODIFIER LETTER ACUTE ACCENT */
/* 131 ID 00a5 */	0x02cb, /* MODIFIER LETTER GRAVE ACCENT */
/* 132 ID 00a6 */	0x0060, /* GRAVE ACCENT */
/* 133 ID 00a7 */	0x02c6, /* MODIFIER LETTER CIRCUMFLEX ACCENT */
/* 134 ID 00a8 */	0x005e, /* CIRCUMFLEX ACCENT */
/* 135 ID 00a9 */	0x00a8, /* DIAERESIS */
/* 136 ID 00aa */	0x0308, /* COMBINING DIAERESIS */
/* 137 ID 00ab */	0x02dc, /* SMALL TILDE */
/* 138 ID 00ac */	0x0303, /* COMBINING TILDE */
/* 139 ID 00ad */	0x02c7, /* CARON */
/* 140 ID 00ae */	0x030c, /* COMBINING CARON */
/* 141 ID 00af */	0x00b8, /* CEDILLA */
/* 142 ID 00b1 */	0x00af, /* MACRON */
/* 143 ID 00b2 */	0x0304, /* COMBINING MACRON */
/* 144 ID 00b3 */	0x02d8, /* BREVE */
/* 145 ID 00b4 */	0x0306, /* COMBINING BREVE */
/* 146 ID 00b5 */	0x02da, /* RING ABOVE */
/* 147 ID 00b6 */	0x030a, /* COMBINING RING ABOVE */
/* 148 ID 00b7 */	0x00c7, /* LATIN CAPITAL LETTER C WITH CEDILLA */
/* 149 ID 00b8 */	0x00e7, /* LATIN SMALL LETTER C WITH CEDILLA */
/* 150 ID 00b9 */	0x215B, /* VULGAR FRACTION ONE EIGHTH 1/8 */
/* 151 ID 00ba */	0x00bc, /* VULGAR FRACTION ONE QUARTER 1/4 */
/* 152 ID 00bb */	0x215c, /* VULGAR FRACTION THREE EIGHTHS 3/8 */
/* 153 ID 00bc */	0x00bd, /* VULGAR FRACTION ONE HALF 1/2 */
/* 154 ID 00bd */	0x215d, /* VULGAR FRACTION FIVE EIGHTHS 5/8 */
/* 155 ID 00be */	0x00be, /* VULGAR FRACTION THREE QUARTERS 3/4 */
/* 156 ID 00bf */	0x215e, /* VULGAR FRACTION SEVEN EIGHTHS 7/8 */
/* 157 ID 00c0 */	0x2153, /* VULGAR FRACTION ONE THIRDS 1/3 */
/* 158 ID 00c1 */	0x2154, /* VULGAR FRACTION TWO THIRDS 2/3 */
/* 159 ID 00c2 */	0x00b9, /* SUPERSCRIPT ONE */
/* 160 ID 00c3 */	0x00b2, /* SUPERSCRIPT TWO */
/* 161 ID 00c4 */	0x00b3, /* SUPERSCRIPT THREE */
/* 162 ID 00c5 */	0x2074, /* SUPERSCRIPT FOUR */
/* 163 ID 00c6 */	0x2075, /* SUPERSCRIPT FIVE */
/* 164 ID 00c7 */	0x2076, /* SUPERSCRIPT SIX */
/* 165 ID 00c8 */	0x2077, /* SUPERSCRIPT SEVEN */
/* 166 ID 00c9 */	0x2078, /* SUPERSCRIPT EIGHT */
/* 167 ID 00ca */	0x2079, /* SUPERSCRIPT NINE */
/* 168 ID 00cb */	0x2070, /* SUPERSCRIPT ZERO */
/* 169 ID 00e1 */	0x00d0, /* LATIN CAPTIAL LETTER ETH */
/* 170 ID 00e2 */	0x0141, /* LATIN CAPITAL LETTER L WITH STROKE */
/* 171 ID 00e3 */	0x0104, /* LATIN CAPITAL LETTER A WITH OGONEK */
/* 172 ID 00e4 */	0x0118, /* LATIN CAPITAL LETTER E WITH OGONEK */
/* 173 ID 00e5 */	0x0111, /* LATIN SMALL LETTER D WITH STROKE */
/* 174 ID 00e6 */	0x0142, /* LATIN SMALL LETTER L WITH STROKE */
/* 175 ID 00e7 */	0x013d, /* LATIN CAPITAL LETTER L WITH CARON */
/* 176 ID 00e8 */	0x0237, /* LATIN SMALL LETTER DOTLESS J */
/* 177 ID 00e9 */	0x0105, /* LATIN SMALL LETTER A WITH OGONEK */
/* 178 ID 00ea */	0x0119, /* LATIN SMALL LETTER E WITH OGONEK */
/* 179 ID 00eb */	-1, /* ligature L.L; no unicode for this */
/* 180 ID 00ec */	-1, /* ligature l.l; no unicode for this */
/* 181 ID 00ed */	0x02d9, /* COMBINING DOT ABOVE */
/* 182 ID 00ee */	0x0308, /* DOT ABOVE */
/* 183 ID 00ef */	0x02dd, /* DOUBLE ACUTE ACCENT */
/* 184 ID 00f0 */	0x030b, /* COMBINING DOUBLE ACUTE ACCENT */
/* 185 ID 00f1 */	0x02bc, /* MODIFIER LETTER APOSTROPHE */
/* 186 ID 00f2 */	0x02cd, /* MODIFIER LETTER LOW MACRON */
/* 187 ID 00f3 */	0x0323, /* COMBINING DOT BELOW */
/* 188 ID 00f4 */	0x0326, /* COMBINING COMMA BELOW */
/* 189 ID 00f5 */	-1, /* COMBINING COMMA BELOW RIGHT */
/* 190 ID 00f6 */	0x02db, /* OGONEK */
/* 191 ID 00f8 */	0x00ad, /* SOFT HYPHEN alt: 0x2012 FIGURE DASH */
/* 192 ID 00f9 */	0x2010, /* HYPHEN */
/* 193 ID 0140 */	0x0144, /* LATIN SMALL LETTER N WITH ACUTE */
/* 194 ID 0141 */	0x0143, /* LATIN CAPITAL LETTER N WITH ACUTE */
/* 195 ID 0142 */	0x00f1, /* LATIN SMALL LETTER N WITH TILDE */
/* 196 ID 0143 */	0x00d1, /* LATIN CAPITAL LETTER N WITH TILDE */
/* 197 ID 0144 */	0x0148, /* LATIN SMALL LETTER N WITH CARON */
/* 198 ID 0145 */	0x0147, /* LATIN CAPITAL LETTER N WITH CARON */
/* 199 ID 0146 */	0x00f3, /* LATIN SMALL LETTER O WITH ACUTE */
/* 200 ID 0147 */	0x00d3, /* LATIN CAPITAL LETTER O WITH ACUTE */
/* 201 ID 0148 */	0x00f2, /* LATIN SMALL LETTER O WITH GRAVE */
/* 202 ID 0149 */	0x00d2, /* LATIN CAPITAL LETTER O WITH ACUTE */
/* 203 ID 014a */	0x00f4, /* LATIN SMALL LETTER O WITH CIRCUMFLEX */
/* 204 ID 014b */	0x00d4, /* LATIN CAPITAL LETTER O WITH CIRCUMFLEX */
/* 205 ID 014c */	0x00f6, /* LATIN SMALL LETTER O WITH DIAERESIS */
/* 206 ID 014d */	0x00d6, /* LATIN CAPITAL LETTER O WITH DIAERESIS */
/* 207 ID 014e */	0x00f5, /* LATIN SMALL LETTER O WITH TILDE */
/* 208 ID 014f */	0x00d5, /* LATIN CAPITAL LETTER O WITH TILDE */
/* 209 ID 0150 */	0x00fa, /* LATIN SMALL LETTER U WITH ACUTE */
/* 210 ID 0151 */	0x00da, /* LATIN CAPITAL LETTER U WITH ACUTE */
/* 211 ID 0152 */	0x00f9, /* LATIN SMALL LETTER U WITH GRAVE */
/* 212 ID 0153 */	0x00d9, /* LATIN CAPITAL LETTER U WITH GRAVE */
/* 213 ID 0154 */	0x00fb, /* LATIN SMALL LETTER U WITH CIRCUMFLEX */
/* 214 ID 0155 */	0x00db, /* LATIN CAPITAL LETTER U WITH CIRCUMFLEX */
/* 215 ID 0156 */	0x00fc, /* LATIN SMALL LETTER U WITH DIAERESIS */
/* 216 ID 0157 */	0x00dc, /* LATIN CAPITAL LETTER U WITH DIAERESIS */
/* 217 ID 0158 */	0x0169, /* LATIN SMALL LETTER U WITH TILDE */
/* 218 ID 0159 */	0x0168, /* LATIN CAPITAL LETTER U WITH TILDE */
/* 219 ID 015a */	0x016f, /* LATIN SMALL LETTER U WITH RING ABOVE */
/* 220 ID 015b */	0x016e, /* LATIN CAPITAL LETTER U WITH RING ABOVE */
/* 221 ID 015c */	0x00ff, /* LATIN SMALL LETTER Y WITH DIAERESIS */
/* 222 ID 015d */	0x0178, /* LATIN CAPITAL LETTER Y WITH DIAERESIS */
/* 223 ID 015e */	0x00fd, /* LATIN SMALL LETTER Y WITH ACUTE */
/* 224 ID 015f */	0x00dd, /* LATIN CAPITAL LETTER Y WITH ACUTE */
/* 225 ID 0161 */	0x00bc, /* VULGAR FRACTION ONE QUARTER 1/4; different glyph than ID 00ba */
/* 226 ID 0162 */	0x00bd, /* VULGAR FRACTION ONE HALF 1/2; different glyph than ID 00bc */
/* 227 ID 0163 */	0x00be, /* VULGAR FRACTION THREE QUARTERS 3/4; different glyph than ID 00be */
/* 228 ID 016a */	0x207e, /* SUPERSCRIPT RIGHT PARENTHESIS */
/* 229 ID 016b */	0x207f, /* SUPERSCRIPT LEFT PARENTHESIS */
/* 230 ID 016e */	0x207b, /* SUPERSCRIPT MINUS */
/* 231 ID 016f */	0x02b9, /* MODIFIER LETTER PRIME */
/* 232 ID 0170 */	0x013e, /* LATIN SMALL LETTER L WITH CARON */
/* 233 ID 0171 */	0x0128, /* LATIN CAPITAL LETTER I WITH TILDE */
/* 234 ID 0172 */	0x0129, /* LATIN SMALL LETTER I WITH TILDE */
/* 235 ID 0173 */	0x00cf, /* LATIN CAPITAL LETTER I WITH DIAERESIS */
/* 236 ID 0174 */	0x00ef, /* LATIN SMALL LETTER I WITH DIAERESIS */
/* 237 ID 0175 */	0x00ce, /* LATIN CAPITAL LETTER I WITH CIRCUMFLEX */
/* 238 ID 0176 */	0x00ee, /* LATIN SMALL LETTER I WITH CIRCUMFLEX */
/* 239 ID 0177 */	0x00cc, /* LATIN CAPITAL LETTER I WITH GRAVE */
/* 240 ID 0178 */	0x00ec, /* LATIN SMALL LETTER I WITH GRAVE */
/* 241 ID 0179 */	0x00cd, /* LATIN CAPITAL LETTER I WITH ACUTE */
/* 242 ID 017a */	0x00ed, /* LATIN SMALL LETTER I WITH ACUTE */
/* 243 ID 017b */	0x011a, /* LATIN CAPITAL LETTER E WITH CARON */
/* 244 ID 017c */	0x011b, /* LATIN SMALL LETTER E WITH CARON */
/* 245 ID 017f */	0x00cb, /* LATIN CAPITAL LETTER E WITH DIAERESIS */
/* 246 ID 0180 */	0x00eb, /* LATIN SMALL LETTER E WITH DIAERESIS */
/* 247 ID 0181 */	0x00ca, /* LATIN CAPITAL LETTER E WITH CIRCUMFLEX */
/* 248 ID 0182 */	0x00ea, /* LATIN SMALL LETTER E WITH CIRCUMFLEX */
/* 249 ID 0183 */	0x00c8, /* LATIN CAPITAL LETTER E WITH GRAVE */
/* 250 ID 0184 */	0x00e8, /* LATIN SMALL LETTER E WITH GRAVE */
/* 251 ID 0185 */	0x00c9, /* LATIN CAPITAL LETTER E WITH ACUTE */
/* 252 ID 0186 */	0x00e9, /* LATIN SMALL LETTER E WITH ACUTE */
/* 253 ID 0187 */	0x00c3, /* LATIN CAPITAL LETTER A WITH TILDE */
/* 254 ID 0188 */	0x00e3, /* LATIN SMALL LETTER A WITH TILDE */
/* 255 ID 0189 */	0x00c4, /* LATIN CAPITAL LETTER A WITH DIAERESIS */
/* 256 ID 018a */	0x00e4, /* LATIN SMALL LETTER A WITH DIAERESIS */
/* 257 ID 018b */	0x00c2, /* LATIN CAPITAL LETTER A WITH CIRCUMFLEX */
/* 258 ID 018c */	0x00e2, /* LATIN SMALL LETTER A WITH CIRCUMFLEX */
/* 259 ID 018d */	0x00c0, /* LATIN CAPITAL LETTER A WITH GRAVE */
/* 260 ID 018e */	0x00e0, /* LATIN SMALL LETTER A WITH GRAVE */
/* 261 ID 018f */	0x00c1, /* LATIN CAPITAL LETTER A WITH ACUTE */
/* 262 ID 0190 */	0x00e1, /* LATIN SMALL LETTER A WITH ACUTE */
/* 263 ID 0194 */	0x0149, /* LATIN SMALL LETTER N PRECEDED BY APOSTROPHE */
/* 264 ID 019a */	0x02c8, /* MODIFIER LETTER VERTICAL LINE */
/* 265 ID 019b */	0x203c, /* DOUBLE EXCLAMATION MARK */
/* 266 ID 019e */	0x20a7, /* PESETA SIGN */
/* 267 ID 01a2 */	0x2014, /* EM DASH */
/* 268 ID 01a5 */	0x0173, /* LATIN SMALL LETTER U WITH OGONEK */
/* 269 ID 01a6 */	0x2019, /* RIGHT SINGLE QUOTATION MARK */
/* 270 ID 01a7 */	0x2026, /* HORIZONTAL ELLIPSIS */
/* 271 ID 01a8 */	0x00de, /* LATIN CAPITAL LETTER THORN */
/* 272 ID 01a9 */	0x00fe, /* LATIN SMALL LETTER THORN */
/* 273 ID 01aa */	0x00f0, /* LATIN SMALL LETTER ETH */
/* 274 ID 01ab */	0x00a5, /* YEN SIGN */
/* 275 ID 01b1 */	0x0133, /* LATIN SMALL LIGATURE IJ */
/* 276 ID 01b2 */	0x0132, /* LATIN CAPITAL LIGATURE IJ */
/* 277 ID 01b3 */	0x00a6, /* BROKEN BAR */
/* 278 ID 01b4 */	0x00a4, /* CURRENCY SIGN */
/* 279 ID 01ba */	0x00b6, /* PILCROW SIGN */
/* 280 ID 01be */	0xfb04, /* LATIN SMALL LIGATURE FFL */
/* 281 ID 01bf */	0xfb03, /* LATIN SMALL LIGATURE FFI */
/* 282 ID 01c0 */	0xfb00, /* LATIN SMALL LIGATURE FF */
/* 283 ID 01c1 */	0x2212, /* MINUS SIGN */
/* 284 ID 01c2 */	0x00d7, /* MULTIPLICATION SIGN */
/* 285 ID 01c3 */	0x00f7, /* DIVISION SIGN */
/* 286 ID 01c4 */	0x00b1, /* PLUS-MINUS SIGN */
/* 287 ID 01c5 */	0x2261, /* IDENTICAL TO */
/* 288 ID 01c6 */	0x2260, /* NOT EQUAL TO */
/* 289 ID 01c7 */	0x2248, /* ALMOST EQUAL TO */
/* 290 ID 01c8 */	0x2265, /* GREATER-THAN OR EQUAL TO */
/* 291 ID 01c9 */	0x2264, /* LESS-THAN OR EQUAL TO */
/* 292 ID 01ca */	0x2193, /* DOWNWARDS ARROW */
/* 293 ID 01cb */	0x2190, /* LEFTWARDS ARROW */
/* 294 ID 01cc */	0x2192, /* RIGHTWARDS ARROW */
/* 295 ID 01cd */	0x2191, /* UPWARDS ARROW */
/* 296 ID 01ce */	0x2195, /* UP DOWN ARROW */
/* 297 ID 01cf */	0x2194, /* LEFT RIGHT ARROW */
/* 298 ID 01d0 */	0x2208, /* ELEMENT OF */
/* 299 ID 01d1 */	0x2229, /* INTERSECTION */
/* 300 ID 01d6 */	0x2320, /* TOP HALF INTEGRAL */
/* 301 ID 01d7 */	0x2321, /* BOTTOM HALF INTEGRAL */
/* 302 ID 01dc */	0x221a, /* SQUARE ROOT */
/* 303 ID 01de */	0x221e, /* INFINITY */
/* 304 ID 01df */	0x2580, /* UPPER HALF BLOCK */
/* 305 ID 01e0 */	0x2584, /* LOWER HAL BLOCK */
/* 306 ID 01e1 */	0x258c, /* LEFT HALF BLOCK */
/* 307 ID 01e2 */	0x2590, /* RIGHT HALF BLOCK */
/* 308 ID 01e4 */	0x2588, /* FULL BLOCK */
/* 309 ID 01e6 */	0x00ac, /* NOT SIGN */
/* 310 ID 01e7 */	0x2310, /* REVERSED NOT SIGN */
/* 311 ID 01e8 */	0x02d7, /* MODIFIER LETTER MINUS SIGN */
/* 312 ID 01e9 */	0x02d6,	/* MODIFIER LETTER PLUS SIGN */
/* 313 ID 01f4 */	0x0393, /* GREEK CAPITAL LETTER GAMMA */
/* 314 ID 01f5 */	0x0394, /* GREEK CAPITAL LETTER DELTA */
/* 315 ID 01f9 */	0x0398, /* GREEK CAPITAL LETTER THETA */
/* 316 ID 0203 */	0x03a3, /* GREEK CAPITAL LETTER SIGMA */
/* 317 ID 0206 */	0x03a6, /* GREEK CAPITAL LETTER PHI */
/* 318 ID 0209 */	0x03a9, /* GREEK CAPITAL LETTER OMEGA */
/* 319 ID 020a */	0x03b1, /* GREEK SMALL LETTER ALPHA */
/* 320 ID 020b */	0x03b2, /* GREEK SMALL LETTER BETA */
/* 321 ID 020e */	0x03b4, /* GREEK SMALL LETTER DELTA */
/* 322 ID 020f */	0x03b5, /* GREEK SMALL LETTER EPSILON */
/* 323 ID 0211 */	0x03b7, /* GREEK SMALL LETTER ETA */
/* 324 ID 0212 */	0x03b8, /* GREEK SMALL LETTER THETA */
/* 325 ID 0216 */	0x00b5, /* MICRO SIGN alt: 0x03bc GREEK SMALL LETTER MU */
/* 326 ID 021a */	0x03c0, /* GREEK SMALL LETTER PI */
/* 327 ID 021c */	0x03c3, /* GREEK SMALL LETTER SIGMA */
/* 328 ID 021e */	0x03c4, /* GREEK SMALL LETTER TAU */
/* 329 ID 0220 */	0x03c6, /* GREEK SMALL LETTER PHI */
/* 330 ID 0232 */	0x266a, /* EIGHTH NOTE */
/* 331 ID 0233 */	0x266b, /* BEAMED EIGHTH NOTES */
/* 332 ID 0234 */	0x00a9, /* COPYRIGHT SIGN; alt: 0x24b8 */
/* 333 ID 0235 */	0x00ae, /* REGISTERED SIGN; alt: 0x24c7 */
/* 334 ID 0236 */	0x2122, /* TRADEMARK */
/* 335 ID 0237 */	0x25a0, /* BLACK SQUARE */
/* 336 ID 0238 */	0x25a1, /* WHITE SQUARE */
/* 337 ID 023a */	0x2032, /* PRIME */
/* 338 ID 023b */	0x2033, /* DOUBLE PRIME */
/* 339 ID 023c */	0x00b0, /* DEGREE */
/* 340 ID 023d */	0x21a8, /* UP DOWN ARROW WITH BASE */
/* 341 ID 023f */	0x2022, /* BULLET */
/* 342 ID 0240 */	0x25cf, /* BLACK CIRCLE */
/* 343 ID 0241 */	0x25ef, /* LARGE CIRCLE */
/* 344 ID 0243 */	0x25d9, /* INVERSE BULLET */
/* 345 ID 0245 */	0x25b6, /* BLACK RIGHT-POINTING TRIANGLE */
/* 346 ID 0246 */	0x25c0, /* BLACK LEFT-POINTING TRIANGLE */
/* 347 ID 0247 */	0x25bc, /* BLACK DOWN-POINTING TRIANGLE */
/* 348 ID 0248 */	0x25b2, /* BLACK UP-POINTING TRIANGLE */
/* 349 ID 0249 */	0x2642, /* MALE SIGN */
/* 350 ID 024a */	0x2640, /* FEMALE SIGN */
/* 351 ID 024b */	0x2665, /* BLACK HEART SUIT */
/* 352 ID 024c */	0x2663, /* BLACK CLUB SUIT */
/* 353 ID 024d */	0x2666, /* BLACK DIAMOND SUIT */
/* 354 ID 024e */	0x2660, /* BLACK SPADE SUIT */
/* 355 ID 024f */	0x2501, /* BOX DRAWINGS HEAVY HORIZONTAL */
/* 356 ID 0250 */	0x2503, /* BOX DRAWINGS HEAVY VERTICAL */
/* 357 ID 0251 */	0x2591, /* LIGHT SHADE */
/* 358 ID 0252 */	0x2592, /* MEDIUM SHADE */
/* 359 ID 0253 */	0x2593, /* DARK SHADE */
/* 360 ID 0256 */	0x263c, /* WHITE SUN WITH RAYS */
/* 361 ID 0257 */	0x263a, /* WHITE SMILING FACE */
/* 362 ID 0258 */	0x263b, /* BLACK SMILING FACE */
/* 363 ID 025d */	0x0165, /* LATIN SMALL LETTER T WITH CARON */
/* 364 ID 025e */	0x0164, /* LATIN CAPITAL LETTER T WITH CARON */
/* 365 ID 025f */	0x0177, /* LATIN SMALL LETTER Y WITH CIRCUMFLEX */
/* 366 ID 0260 */	0x0176, /* LATIN CAPITAL LETTER Y WITH CIRCUMFLEX */
/* 367 ID 0261 */	0x017a, /* LATIN SMALL LETTER Z WITH ACUTE */
/* 368 ID 0262 */	0x0179, /* LATIN CAPITAL LETTER Z WITH ACUTE */
/* 369 ID 0263 */	0x017e, /* LATIN SMALL LETTER Z WITH CARON */
/* 370 ID 0264 */	0x017d, /* LATIN CAPITAL LETTER Z WITH CARON */
/* 371 ID 0265 */	0x017c, /* LATIN SMALL LETTER Z WITH DOT ABOVE */
/* 372 ID 0266 */	0x017b, /* LATIN CAPITAL LETTER Z WITH DOT ABOVE */
/* 373 ID 026b */	0x0103, /* LATIN SMALL LETTER A WITH BREVE */
/* 374 ID 026c */	0x0102, /* LATIN CAPITAL LETTER A WITH BREVE */
/* 375 ID 026d */	0x0107, /* LATIN SMALL LETTER C WITH ACUTE */
/* 376 ID 026e */	0x0106, /* LATIN CAPITAL LETTER C WITH ACUTE */
/* 377 ID 026f */	0x010d, /* LATIN SMALL LETTER C WITH CARON */
/* 378 ID 0270 */	0x010c, /* LATIN CAPITAL LETTER C WITH CARON */
/* 379 ID 0271 */	0x010f, /* LATIN SMALL LETTER D WITH CARON */
/* 380 ID 0274 */	0x018c,	/* LATIN SMALL LETTER D WITH TOPBAR */
/* 381 ID 0275 */	0x018b,	/* LATIN CAPITAL LETTER D WITH TOPBAR */
/* 382 ID 0276 */	0x0113, /* LATIN SMALL LETTER E WITH MACRON */
/* 383 ID 0277 */	0x0112, /* LATIN CAPITAL LETTER E WITH MACRON */
/* 384 ID 027a */	0x01f5, /* LATIN SMALL LETTER G WITH ACUTE */
/* 385 ID 027d */	0x0122, /* LATIN CAPITAL LETTER G WITH CEDILLA */
/* 386 ID 027e */	0x012b, /* LATIN SMALL LETTER I WITH MACRON */
/* 387 ID 027f */	0x012a, /* LATIN CAPITAL LETTER I WITH MACRON */
/* 388 ID 0280 */	0x0131, /* LATIN SMALL LETTER DOTLESS I */
/* 389 ID 0281 */	0x0130, /* LATIN CAPITAL LETTER I WITH DOT ABOVE */
/* 390 ID 0282 */	0x012f, /* LATIN SMALL LETTER I WITH OGONEK */
/* 391 ID 0283 */	0x012e, /* LATIN CAPITAL LETTER I WITH OGONEK */
/* 392 ID 0284 */	0x0137, /* LATIN SMALL LETTER K WITH CEDILLA */
/* 393 ID 0285 */	0x0136, /* LATIN CAPITAL LETTER K WITH CEDILLA */
/* 394 ID 0286 */	0x013a, /* LATIN SMALL LETTER L WITH ACUTE */
/* 395 ID 0287 */	0x0139, /* LATIN CAPITAL LETTER L WITH ACUTE */
/* 396 ID 0288 */	-1, /* 006c 0304 LATIN SMALL LETTER L WITH MACRON; no unicode for this */
/* 397 ID 0289 */	-1, /* 004c 0304 LATIN CAPITAL LETTER L WITH MACRON; no unicode for this */
/* 398 ID 028a */	0x013c, /* LATIN SMALL LETTER L WITH CEDILLA */
/* 399 ID 028b */	0x013b, /* LATIN CAPITAL LETTER L WITH CEDILLA */
/* 400 ID 028d */	-1, /* 006c 0304 LATIN CAPITAL LETTER N WITH MACRON; no unicode for this */
/* 401 ID 028e */	0x0146, /* LATIN SMALL LETTER N WITH CEDILLA */
/* 402 ID 028f */	0x0145, /* LATIN CAPITAL LETTER N WITH CEDILLA */
/* 403 ID 0290 */	0x0151, /* LATIN SMALL LETTER O WITH DOUBLE ACUTE */
/* 404 ID 0291 */	0x0150, /* LATIN CAPITAL LETTER O WITH DOUBLE ACUTE */
/* 405 ID 0294 */	0x0159, /* LATIN SMALL LETTER R WITH CARON */
/* 406 ID 0295 */	0x0158, /* LATIN CAPITAL LETTER R WITH CARON */
/* 407 ID 0296 */	0x0157, /* LATIN SMALL LETTER R WITH CEDILLA */
/* 408 ID 0297 */	0x0156, /* LATIN CAPITAL LETTER R WITH CEDILLA */
/* 409 ID 0298 */	0x015b, /* LATIN SMALL LETTER S WITH ACUTE */
/* 410 ID 0299 */	0x015a, /* LATIN CPAITAL LETTER S WITH ACUTE */
/* 411 ID 029a */	0x0161, /* LATIN SMALL LETTER S WITH CARON */
/* 412 ID 029b */	0x0160, /* LATIN CAPITAL LETTER S WITH CARON */
/* 413 ID 029d */	-1, /* 0053 0304 LATIN CAPITAL LETTER S WITH MACRON; no unicode for this */
/* 414 ID 029e */	0x015f,	/* LATIN SMALL LETTER S WITH CEDILLA */
/* 415 ID 029f */	0x015e,	/* LATIN CAPITAL LETTER S WITH CEDILLA */
/* 416 ID 02a2 */	-1, /* 0073 0304 LATIN SMALL LETTER T WITH MACRON; no unicode for this */
/* 417 ID 02a3 */	-1, /* 0053 0304 LATIN CAPITAL LETTER T WITH MACRON; no unicode for this */
/* 418 ID 02a4 */	0x0163, /* LATIN SMALL LETTER T WITH CEDILLA */
/* 419 ID 02a5 */	0x0162, /* LATIN CAPITAL LETTER T WITH CEDILLA */
/* 420 ID 02a6 */	0x016b, /* LATIN SMALL LETTER U WITH MACRON */
/* 421 ID 02a7 */	0x016a, /* LATIN CAPITAL LETTER U WITH MACRON */
/* 422 ID 02a8 */	0x0171, /* LATIN SMALL LETTER U WITH DOUBLE ACUTE */
/* 423 ID 02a9 */	0x0170, /* LATIN CAPITAL LETTER U WITH DOUBLE ACUTE */
/* 424 ID 02aa */	0x0175, /* LATIN SMALL LETTER W WITH CIRCUMFLEX */
/* 425 ID 02ab */	0x0174, /* LATIN CAPITAL LETTER W WITH CIRCUMFLEX */
/* 426 ID 02ac */	-1,	/* 0x0079 0x030c LATIN SMALL LETTER Y WITH CARON; no unicode for this */
/* 427 ID 02ad */	-1,	/* 0x0059 0x030c LATIN CAPITAL LETTER Y WITH CARON; no unicode for this */
/* 428 ID 02b5 */	0x222e, /* CONTOUR INTEGRAL */
/* 429 ID 02b7 */	0x221d, /* PROPORTIONAL TO */
/* 430 ID 031d */	0x2510, /* BOX DRAWINGS LIGHT DOWN AND LEFT */
/* 431 ID 04c7 */	0x2044, /* FRACTION SLASH */
/* 432 ID 0554 */	0x0301, /* COMBINING ACUTE ACCENT */
/* 433 ID 0555 */	0x0341, /* COMBINING ACUTE TONE MARK */
/* 434 ID 0558 */	0x0300, /* COMBINING GRAVE ACCENT */
/* 435 ID 0559 */	0x0340, /* COMBINING GRAVE TONE MARK */
/* 436 ID 055c */	0x0302, /* COMBINING CIRCUMFLEX ACCENT */
/* 437 ID 055d */	-1, /* COMBINING CIRCUMFLEX MARK; no unicode for this */
/* 438 ID 0560 */	0x0308, /* COMBINING DIAERESIS */
/* 439 ID 0561 */	-1, /* COMBINING DIAERESIS MARK; no unicode for this */
/* 440 ID 0564 */	0x0303, /* COMBINING TILDE */
/* 441 ID 0565 */	0x0334, /* COMBINING TILDE OVERLAY */
/* 442 ID 0568 */	0x030c, /* COMBINING CARON */
/* 443 ID 0569 */	-1, /* COMBINING CARON MARK; no unicode for this */
/* 444 ID 056c */	0x0327, /* COMBINING CEDILLA */
/* 445 ID 0570 */	0x0304, /* MACRON */
/* 446 ID 0571 */	0x00af, /* COMBINING MACRON */
/* 447 ID 0574 */	0x0306, /* COMBINING BREVE */
/* 448 ID 0575 */	0x02d8, /* BREVE */
/* 449 ID 0578 */	0x030a, /* COMBINING RING ABOVE */
/* 450 ID 0579 */	-1, /* COMBINING RING ABOVE; no unicode for this */
/* 451 ID 067d */	0x013f, /* LATIN CAPITAL LETTER L WITH MIDDLE DOT */
/* 452 ID 0683 */	0x0140, /* LATIN SMALL LETTER L WITH MIDDLE DOT */
/* 453 ID 06cf */	0x0307, /* COMBINING DOT ABOVE */
/* 454 ID 06d0 */	0x0358, /* COMBINING DOT ABOVE RIGHT */
/* 455 ID 06d3 */	0x030b, /* COMBINING DOUBLE ACUTE ACCENT */
/* 456 ID 06d4 */	0x030b, /* COMBINING DOUBLE ACUTE ACCENT */
/* 457 ID 06d7 */	0x0313, /* COMBINING COMMA ABOVE */
/* 458 ID 06d8 */	0x0315, /* COMBINING COMMA RIGHT */
/* 459 ID 06d9 */	0x0315, /* COMBINING COMMA RIGHT */
/* 460 ID 06dc */	0x0331, /* COMBINING MACRON BELOW */
/* 461 ID 06e1 */	0x0323, /* COMBINING DOT BELOW */
/* 462 ID 06e6 */	0x0326, /* COMBINING DOT BELOW */
/* 463 ID 06eb */	0x0314, /* COMBINING REVERSED COMMA BELOW */
/* 464 ID 06f0 */	0x06f0, /* COMBINING OGONEK */
/* 465 ID 07cc */	0x203e, /* OVERLINE */
/* 466 ID 07e6 */	0x2081, /* SUBSCRIPT ONE */
/* 467 ID 07ec */	0x2082, /* SUBSCRIPT TWO */
/* 468 ID 07f2 */	0x2083, /* SUBSCRIPT THREE */
/* 469 ID 07f8 */	0x2084, /* SUBSCRIPT FOUR */
/* 470 ID 07fe */	0x2085, /* SUBSCRIPT FIVE */
/* 471 ID 0804 */	0x2086, /* SUBSCRIPT SIX */
/* 472 ID 080a */	0x2087, /* SUBSCRIPT SEVEN */
/* 473 ID 0810 */	0x2088, /* SUBSCRIPT EIGHT */
/* 474 ID 0816 */	0x2089, /* SUBSCRIPT NINE */
/* 475 ID 081c */	0x2080, /* SUBSCRIPT ZERO */
/* 476 ID 0a57 */	0x0101, /* LATIN SMALL LETTER A WITH MACRON */
/* 477 ID 0a5d */	0x0100, /* LATIN CAPITAL LETTER A WITH MACRON */
/* 478 ID 0ad8 */	0x010e, /* LATIN CAPITAL LETTER D WITH CARON */
/* 479 ID 0ba8 */	0x01e7, /* LATIN SMALL LETTER G WITH CARON */
/* 480 ID 0bae */	0x01e6, /* LATIN CAPITAL LETTER G WITH CARON */
/* 481 ID 0d44 */	-1, /* LATIN SMALL LETTER N WITH MACRON; no unicode for this */
/* 482 ID 0dfc */	-1, /* LATIN SMALL LETTER R WITH GRAVE; no unicode for this */
/* 483 ID 0e02 */	-1, /* LATIN CAPITAL LETTER R WITH GRAVE; no unicode for this */
/* 484 ID 0e78 */	-1, /* LATIN SMALL LETTER S WITH MACRON; no unicode for this */
/* 485 ID 0e9a */	0x015f, /* LATIN SMALL LETTER S WITH CEDILLA */
/* 486 ID 0ea0 */	0x015e, /* LATIN CAPITAL LETTER S WITH CEDILLA */
/* 487 ID 1178 */	0x2017, /* DOUBLE LOW LINE */
/* 488 ID 1188 */	0x0313, /* COMBINING COMMA ABOVE */
/* 489 ID 1189 */	0x0313, /* COMBINING COMMA ABOVE */
/* 490 ID 118a */	0x0313, /* COMBINING COMMA ABOVE */
/* 491 ID 11ac */	0x20a3, /* FRENCH FRANC SIGN */
/* 492 ID 1280 */	0x2213, /* MINUS-OR-PLUS SIGN */
/* 493 ID 1288 */	0x223c, /* TILDE OPERATOR */
/* 494 ID 1327 */	0x2514, /* BOX DRAWINGS LIGHT UP AND RIGHT */
/* 495 ID 13b2 */	0x2b24, /* BLACK LARGE CIRCLE */
/* 496 ID 13dd */	0x2502, /* BOX DRAWINGS LIGHT VERTICAL */
/* 497 ID 141b */	-1, /* looks like hand-written l */
/* 498 ID 144c */	0x2b1b, /* BLACK LARGE SQUARE */
/* 499 ID 147b */	0x2218, /* RING OPERATOR */
/* 500 ID 147c */	0x2219, /* BULLET OPERATOR */
/* 501 ID 1481 */	0x221f, /* RIGHT ANGLE */
/* 502 ID 148e */	0x025b, /* LATIN SMALL LETTER OPEN E (epsilon) */
/* 503 ID 14fb */	0x00a9, /* COPYRIGHT */
/* 504 ID 14fc */	0x00ae, /* REGISTERED */
/* 505 ID 151b */	0x25cb, /* WHITE CIRCLE */
/* 506 ID 1520 */	0x25d9, /* INVERSE WHITE CIRCLE */
/* 507 ID 1522 */	0x25aa, /* BLACK SMALL SQUARE */
/* 508 ID 152a */	0x2302, /* HOUSE */
/* 509 ID 152d */	0x2500, /* BOX DRAWINGS LIGHT HORIZONTAL */
/* 510 ID 152e */	0x250c, /* BOX DRAWINGS LIGHT DOWN AND RIGHT */
/* 511 ID 152f */	0x2518, /* BOX DRAWINGS LIGHT UP AND LEFT */
/* 512 ID 1530 */	0x2514, /* BOX DRAWINGS LIGHT UP AND RIGHT */
/* 513 ID 1533 */	0x253c, /* BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL */
/* 514 ID 1534 */	0x252c, /* BOX DRAWINGS LIGHT DOWN AND HORIZONTAL */
/* 515 ID 1535 */	0x2534, /* BOX DRAWINGS LIGHT UP AND HORIZONTAL */
/* 516 ID 1536 */	0x251c, /* BOX DRAWINGS LIGHT VERTICAL AND RIGHT */
/* 517 ID 1537 */	0x2524, /* BOX DRAWINGS LIGHT VERTICAL AND LEFT */
/* 518 ID 1538 */	0x2502, /* BOX DRAWINGS LIGHT VERTICAL */
/* 519 ID 153a */	0x2550, /* BOX DRAWINGS DOUBLE HORIZONTAL */
/* 520 ID 153b */	0x2557, /* BOX DRAWINGS DOUBLE DOWN AND LEFT */
/* 521 ID 153c */	0x2554, /* BOX DRAWINGS DOUBLE DOWN AND RIGHT */
/* 522 ID 153d */	0x255d, /* BOX DRAWINGS DOUBLE UP AND LEFT */
/* 523 ID 153e */	0x255a, /* BOX DRAWINGS DOUBLE UP AND RIGHT */
/* 524 ID 1541 */	0x256c, /* BOX DRAWINGS DOUBLE VERTICAL AND HORIZONTAL */
/* 525 ID 1542 */	0x2566, /* BOX DRAWINGS DOUBLE DOWN AND HORIZONTAL */
/* 526 ID 1543 */	0x2569, /* BOX DRAWINGS DOUBLE UP AND HORIZONTAL */
/* 527 ID 1544 */	0x2560, /* BOX DRAWINGS DOUBLE VERTICAL AND RIGHT */
/* 528 ID 1545 */	0x2563, /* BOX DRAWINGS DOUBLE VERTICAL AND LEFT */
/* 529 ID 1546 */	0x2551, /* BOX DRAWINGS DOUBLE VERTICAL */
/* 530 ID 1555 */	0x255e, /* BOX DRAWINGS VERTICAL SINGLE AND RIGHT DOUBLE */
/* 531 ID 1556 */	0x2561, /* BOX DRAWINGS VERTICAL SINGLE AND LEFT DOUBLE */
/* 532 ID 1557 */	0x2568, /* BOX DRAWINGS UP DOUBLE AND HORIZONTAL SINGLE */
/* 533 ID 1558 */	0x2565, /* BOX DRAWINGS DOWN DOUBLE AND HORIZONTAL SINGLE */
/* 534 ID 1559 */	0x256a, /* BOX DRAWINGS VERTICAL SINGLE AND HORIZONTAL DOUBLE */
/* 535 ID 155a */	0x256b, /* BOX DRAWINGS VERTICAL DOUBLE AND HORIZONTAL SINGLE */
/* 536 ID 155b */	0x255f, /* BOX DRAWINGS VERTICAL DOUBLE AND RIGHT SINGLE */
/* 537 ID 155c */	0x2562, /* BOX DRAWINGS VERTICAL DOUBLE AND LEFT SINGLE */
/* 538 ID 1586 */	0x2090, /* LATIN SUBSCRIPT SMALL LETTER A */
/* 539 ID 158a */	0x2091, /* LATIN SUBSCRIPT SMALL LETTER E */
/* 540 ID 158e */	0x1d62, /* LATIN SUBSCRIPT SMALL LETTER I */
/* 541 ID 1591 */	0x2097, /* LATIN SUBSCRIPT SMALL LETTER L */
/* 542 ID 1592 */	0x2098, /* LATIN SUBSCRIPT SMALL LETTER M */
/* 543 ID 1593 */	0x2099, /* LATIN SUBSCRIPT SMALL LETTER N */
/* 544 ID 1594 */	0x2092, /* LATIN SUBSCRIPT SMALL LETTER O */
/* 545 ID 1597 */	0x1d63, /* LATIN SUBSCRIPT SMALL LETTER R */
/* 546 ID 1598 */	0x209b, /* LATIN SUBSCRIPT SMALL LETTER S */
/* 547 ID 1599 */	0x209c, /* LATIN SUBSCRIPT SMALL LETTER T */
/* 548 ID 15a0 */	0x2567, /* BOX DRAWINGS UP SINGLE AND HORIZONTAL DOUBLE */
/* 549 ID 15a1 */	0x2564, /* BOX DRAWINGS DOWN SINGLE AND HORIZONTAL DOUBLE */
/* 550 ID 15a2 */	0x2556, /* BOX DRAWINGS DOWN DOUBLE AND LEFT SINGLE */
/* 551 ID 15a3 */	0x2555, /* BOX DRAWINGS DOWN SINGLE AND LEFT DOUBLE */
/* 552 ID 15a4 */	0x255c, /* BOX DRAWINGS UP DOUBLE AND LEFT SINGLE */
/* 553 ID 15a5 */	0x255b, /* BOX DRAWINGS UP SINGLE AND LEFT DOUBLE */
/* 554 ID 15a6 */	0x2559, /* BOX DRAWINGS UP DOUBLE AND RIGHT SINGLE */
/* 555 ID 15a7 */	0x2558, /* BOX DRAWINGS UP SINGLE AND RIGHT DOUBLE */
/* 556 ID 15a8 */	0x2553, /* BOX DRAWINGS DOWN DOUBLE AND RIGHT SINGLE */
/* 557 ID 15a9 */	0x2552, /* BOX DRAWINGS DOWN SINGLE AND RIGHT DOUBLE */
/* 558 ID 15ac */	0x2572, /* BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO LOWER RIGHT */
/* 559 ID 15b2 */	0x2311, /* SQUARE LOZENGE */
/* 560 ID 15da */	0x00a0, /* NO-BREAK SPACE */
/* 561 ID 15db */	-1,
/* 562 ID 15dc */	-1,
/* 563 ID 193a */	-1 /* smiley with tongue */
};

#if 0
/*     ID 01f2 */	0x0391, /* GREEK CAPITAL LETTER ALPHA */
/*     ID 01f3 */	0x0392, /* GREEK CAPITAL LETTER BETA */
/*     ID 01f6 */	0x0395, /* GREEK CAPITAL LETTER EPSILON */
/*     ID 01f7 */	0x0396, /* GREEK CAPITAL LETTER ZETA */
/*     ID 01f8 */	0x0397, /* GREEK CAPITAL LETTER ETA */
/*     ID 01fa */	0x0399, /* GREEK CAPITAL LETTER IOTA */
/*     ID 01fb */	0x039a, /* GREEK CAPITAL LETTER KAPPA */
/*     ID 01fc */	0x039b, /* GREEK CAPITAL LETTER LAMBDA */
/*     ID 01fd */	0x039c, /* GREEK CAPITAL LETTER MU */
/*     ID 01fe */	0x039d, /* GREEK CAPITAL LETTER NU */
/*     ID 01ff */	0x039e, /* GREEK CAPITAL LETTER XI */
/*     ID 0200 */	0x039f, /* GREEK CAPITAL LETTER OMICRON */
/*     ID 0201 */	0x03a0, /* GREEK CAPITAL LETTER PI */
/*     ID 0202 */	0x03a1, /* GREEK CAPITAL LETTER RHO */
/*     ID 0204 */	0x03a4, /* GREEK CAPITAL LETTER TAU */
/*     ID 0205 */	0x03a5, /* GREEK CAPITAL LETTER UPSILON */
/*     ID 0207 */	0x03a7, /* GREEK CAPITAL LETTER CHI */
/*     ID 0208 */	0x03a8, /* GREEK CAPITAL LETTER PSI */
/*     ID 020d */	0x03b3, /* GREEK SMALL LETTER GAMMA */
/*     ID 0210 */	0x03b6, /* GREEK SMALL LETTER ZETA */
/*     ID 0213 */	0x03b9, /* GREEK SMALL LETTER IOTA */
/*     ID 0214 */	0x03ba, /* GREEK SMALL LETTER KAPPA */
/*     ID 0215 */	0x03bb, /* GREEK SMALL LETTER LAMBDA */
/*     ID 0217 */	0x03bd, /* GREEK SMALL LETTER NU */
/*     ID 0218 */	0x03be, /* GREEK SMALL LETTER XI */
/*     ID 0219 */	0x03bf, /* GREEK SMALL LETTER OMICRON */
/*     ID 021b */	0x03c1, /* GREEK SMALL LETTER RHO */
/*     ID 021d */	0x03c2, /* GREEK SMALL LETTER FINAL SIGMA */
/*     ID 021f */	0x03c5, /* GREEK SMALL LETTER UPSILON */
/*     ID 0221 */	0x03c7, /* GREEK SMALL LETTER CHI */
/*     ID 0222 */	0x03c8, /* GREEK SMALL LETTER PSI */
/*     ID 0223 */	0x03c9, /* GREEK SMALL LETTER OMEGA */
#endif

static short const table_00a0[] = {
	560, 128,  98,  97, 278, 274, 277, 110, 135, 503, 538, 125, 309, 191, 504, 230,
	339, 286, 160, 161, 129, 325, 279, 102, 141, 159, 544, 126, 151, 153, 155, 127,
	259, 261, 257, 253, 255, 113, 114, 148, 249, 251, 247, 245, 239, 241, 237, 235,
	169, 196, 202, 200, 204, 208, 206, 284, 115, 212, 210, 214, 216, 224, 271, 121,
	260, 262, 258, 254, 256, 117, 118, 149, 250, 252, 248, 246, 240, 242, 238, 236,
	273, 195, 201, 199, 203, 207, 205, 285, 119, 211, 209, 213, 215, 223, 272, 221,
	477, 476, 374, 373, 171, 177, 376, 375,  -1,  -1,  -1,  -1, 378, 377,  -1, 379,
	169, 173, 383, 382,  -1,  -1,  -1,  -1, 172, 178, 243, 244,  -1,  -1,  -1,  -1,
	 -1,  -1, 385,  -1,  -1,  -1,  -1,  -1, 233, 234, 387, 386,  -1,  -1, 391, 390,
	389, 122, 276, 275,  -1,  -1, 393, 392,  -1, 395, 394, 399, 398,  -1,  -1,  -1,
	 -1, 170, 174, 194, 193, 402, 401, 198, 197, 263,  -1,  -1,  -1,  -1,  -1,  -1,
	404, 403, 116, 120,  -1,  -1, 408, 407, 406, 405, 410, 409,  -1,  -1, 486, 485,
	412, 411, 419, 418, 364, 363,  -1,  -1, 218, 217, 421, 420,  -1,  -1, 220, 219,
	423, 422,  -1, 268, 425, 424,  -1,  -1, 222, 368, 367, 372, 371, 370, 369
};

static short const table_02d8[] = { 144, 181, 146, 190, 137, 183 };

static short const table_0393[] = {
	313, 314,  -1,  -1,  -1, 315,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
	316,  -1,  -1, 317,  -1,  -1, 318,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 319, 320,
	 -1, 321, 322,  -1, 323, 324,  -1,  -1,  -1, 325,  -1,  -1,  -1, 326,  -1,  -1,
	327, 328,  -1, 329
};

static short const table_2013[] = {
	111, 112,  -1,  -1,  -1, 103,  -1, 106,  -1, 104, 105, 107,  -1, 108, 109,  -1,
	 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 100,  -1,  -1,
	 -1,  -1,  -1,  -1,  -1,  -1, 123, 124,  -1, 265
};

static short const table_207f[] = {
	 -1, 475, 466, 467, 468, 469, 470, 471, 472, 473, 474,  -1,  -1,  -1,  -1,  -1,
	 -1, 538,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 543,  -1,  -1,  -1,  -1,  -1,
	 -1,  -1,  -1,  -1, 491,  -1,  -1,  -1, 266
};

static short const table_2190[] = { 293, 295, 294, 292, 297, 296 };

static short const table_2208[] = {
	298,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 492,  -1,  -1,  -1,  -1,
	 -1,  -1, 302,  -1,  -1,  -1, 303,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
	 -1, 299,  -1,  -1,  -1,  -1, 428
};


static short const table_2260[] = { 288,  -1,  -1,  -1, 291, 290 };

static short const table_2580[] = {
	304,  -1,  -1,  -1, 305,  -1,  -1,  -1, 308,  -1,  -1,  -1, 306,  -1,  -1,  -1,
	307, 357, 358, 359,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
	335, 336,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
	 -1,  -1, 348,  -1,  -1,  -1, 345,  -1,  -1,  -1,  -1,  -1, 347,  -1,  -1,  -1,
	346,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 339,  -1,  -1,  -1, 342,
	 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 344
};

static short const table_2638[] = {  -1,  -1, 361, 362, 360,  -1,  -1,  -1, 350,  -1, 349 };

static short const table_2660[] = { 354,  -1,  -1, 352,  -1, 351, 353,  -1,  -1,  -1, 330, 331 };

static short const table_fb00[] = { 282, 95, 96, 281, 280 };

int unicode_to_bics(unsigned int code)
{
	if (code < 0x20)
		return -1;
	if (code < 0x7f)
		return code - 0x20;
	if (code < 0x00a0)
		return -1;
	if (code < 0x017f)
	{
		verify((sizeof(table_00a0) / sizeof(table_00a0[0])) == (0x017f - 0x00a0));
		return table_00a0[code - 0x00a0];
	}
	if (code == 0x0192)
		return 99;
	if (code == 0x01e6)
		return 480;
	if (code == 0x01e7)
		return 379;
	if (code == 0x01f5)
		return 384;
	if (code == 0x02c7)
		return 139;
	if (code < 0x02d8)
		return -1;
	if (code < 0x02de)
	{
		verify((sizeof(table_02d8) / sizeof(table_02d8[0])) == (0x02de - 0x02d8));
		return table_02d8[code - 0x02d8];
	}
	if (code < 0x0393)
		return -1;
	if (code < 0x03c7)
	{
		verify((sizeof(table_0393) / sizeof(table_0393[0])) == (0x03c7 - 0x0393));
		return table_0393[code - 0x0393];
	}
	if (code < 0x2013)
		return -1;
	if (code < 0x203d)
	{
		verify((sizeof(table_2013) / sizeof(table_2013[0])) == (0x203d - 0x2013));
		return table_2013[code - 0x2013];
	}
	if (code == 0x207b)
		return 230;
	if (code < 0x207f)
		return -1;
	if (code < 0x20a8)
	{
		verify((sizeof(table_207f) / sizeof(table_207f[0])) == (0x20a8 - 0x207f));
		return table_207f[code - 0x207f];
	}
	if (code < 0x2190)
		return -1;
	if (code < 0x2196)
	{
		verify((sizeof(table_2190) / sizeof(table_2190[0])) == (0x2196 - 0x2190));
		return table_2190[code - 0x2190];
	}
	if (code == 0x21a8)
		return 340;
	if (code < 0x2208)
		return -1;
	if (code < 0x222f)
	{
		verify((sizeof(table_2208) / sizeof(table_2208[0])) == (0x222f - 0x2208));
		return table_2208[code - 0x2208];
	}
	if (code < 0x2260)
		return -1;
	if (code < 0x2266)
	{
		verify((sizeof(table_2260) / sizeof(table_2260[0])) == (0x2266 - 0x2260));
		return table_2260[code - 0x2260];
	}
	if (code == 0x2310)
		return 310;
	if (code == 0x2320)
		return 300;
	if (code == 0x2321)
		return 301;
	if (code == 0x24b8)
		return 332;
	if (code == 0x24c7)
		return 333;
	if (code == 0x2501)
		return 355;
	if (code == 0x2503)
		return 356;
	if (code == 0x2558)
		return 555;
	if (code < 0x2580)
		return -1;
	if (code < 0x25da)
	{
		verify((sizeof(table_2580) / sizeof(table_2580[0])) == (0x25da - 0x2580));
		return table_2580[code - 0x2580];
	}
	if (code == 0x25ef)
		return 343;
	if (code < 0x2638)
		return -1;
	if (code < 0x2643)
	{
		verify((sizeof(table_2638) / sizeof(table_2638[0])) == (0x2643 - 0x2638));
		return table_2638[code - 0x2638];
	}
	if (code < 0x2660)
		return -1;
	if (code < 0x266c)
	{
		verify((sizeof(table_2660) / sizeof(table_2660[0])) == (0x266c - 0x2660));
		return table_2660[code - 0x2660];
	}
	if (code < 0xfb00)
		return -1;
	if (code < 0xfb05)
	{
		verify((sizeof(table_fb00) / sizeof(table_fb00[0])) == (0xfb05 - 0xfb00));
		return table_fb00[code - 0xfb00];
	}
	return -1;
}


#ifdef MAIN

#include <stdio.h>

int main(void)
{
	unsigned int i;
	int bics;
	int j;
	
	for (i = 0; i < 0xffff; i++)
	{
		bics = unicode_to_bics(i);
		if (bics >= 0)
		{
			if (bics >= BICS_COUNT)
			{
				printf("%d: %04x: too high\n", bics, i);
			} else
			{
				unsigned int code = Bics2Unicode[bics];
				if (code != i)
				{
					printf("%d: %04x != %04x", bics, code, i);
					for (j = 0; j < BICS_COUNT; j++)
					{
						if (Bics2Unicode[j] == i)
						{
							printf(" (duplicate: %d)", j);
						}
					}
					printf("\n");
				}
			}
		}
	}
	return 0;
}
#endif
